﻿;----------------------------------------------------------------
; Autohotkey MDI Application Example
; DDART, 2020.12.27
; https://auto.ddart.net/xe/autohotkey/2036
;----------------------------------------------------------------
#Persistent
#SingleInstance Force
#include <windows>
class CMDIFrameWindow extends CFrameWindow
{
	SetCreateWindowInfo()
	{
		base.SetCreateWindowInfo()
		this.cwex.dwStyle |= CCWEX.WS_CLIPCHILDREN
	}
	AddMenu()
	{
		this.ID_FILE_NEW := this.id_last++
		this.ID_FILE_CLOSE := this.id_last++
		this.ID_FILE_EXIT := this.id_last++
		file := new CMenu()
		file.AppendMenu(CMenu.MF_STRING, this.ID_FILE_NEW, "새로 만들기(&N)")
		file.AppendMenu(CMenu.MF_STRING, this.ID_FILE_CLOSE, "닫 기(&C)")
		file.AppendMenu(CMenu.MF_STRING, this.ID_FILE_EXIT, "종 료(&X)")
		this.menu.AppendMenu(CMenu.MF_POPUP, file.hMenu, "파일(&F)")
		window := new CMenu()
		this.ID_WINDOW_CASCADE := this.id_last++
		this.ID_WINDOW_TILE := this.id_last++
		this.ID_WINDOW_ICONARRANGE := this.id_last++
		this.ID_WINDOW_CLOSEALL := this.id_last++
		window.AppendMenu(CMenu.MF_STRING, this.ID_WINDOW_CASCADE, "계단식 정렬(&C)")
		window.AppendMenu(CMenu.MF_STRING, this.ID_WINDOW_TILE, "타일식 정렬(&T)")
		window.AppendMenu(CMenu.MF_STRING, this.ID_WINDOW_ICONARRANGE, "아이콘 정렬 (&I)")
		window.AppendMenu(CMenu.MF_STRING, this.ID_WINDOW_CLOSEALL, "모두 닫기 (&A)")
		this.menu.AppendMenu(CMenu.MF_POPUP, window.hMenu, "창(&W)")
	}
	OnCreate(hWnd, uMsg, wParam, lParam)
	{
		TRACE(A_LineFile, A_LineNumber, A_Thisfunc)
		hWindowMenu := this.menu.GetSubMenu(1)
		this.mdiclient := new CMDIClientWindow(hWnd, hWindowMenu)
		this.mdiclient.CreateEx()
		return 0
	}
	OnCommand(hWnd, uMsg, wParam, lParam)
	{
		id := wParam & 0xFFFF
		if (id = this.ID_FILE_NEW)
		{
			static new_count := 1
			this.mdiclient.CreateChild("Autohotkey MDI Child " new_count++)
			;this.mdiclient.SetMenu(this.menu.hMenu, hWindowMenu)
			;this.mdiclient.RefreshMenu()
			return true
		}
		else if (id = this.ID_FILE_CLOSE)
		{
			child := new CWindow(this.mdiclient.GetActive())
			LogMessage(child.hWnd)
			if (child.IsWindow() && child.SendMessage(CWindowMessage.WM_QUERYENDSESSION, 0, 0))
			{
				this.mdiclient.Destory(child.hWnd)
			}
			return true
		}
		else if (id = this.ID_WINDOW_CASCADE)
		{
			this.mdiclient.Cascade()
			return true
		}
		else if (id = this.ID_WINDOW_TILE)
		{
			this.mdiclient.Tile()
			return true
		}
		else if (id = this.ID_WINDOW_ICONARRANGE)
		{
			this.mdiclient.IconArrange()
			return true
		}
		else if (id = this.ID_WINDOW_CLOSEALL)
		{
			this.mdiclient.CloseAll()
			return true
		}
		else
		{
			child := new CWindow(this.mdiclient.GetActive())
			if (child.IsWindow())
			{
				child.SendMessage(uMsg, wParam, lParam)
			}
		}
		return base.OnCommand(hWnd, uMsg, wParam, lParam)
	}
	OnMessage(hWnd, uMsg, wParam, lParam)
	{
		static __ := CWindowMessage
		if (uMsg = __.WM_CLOSE || uMsg = __.WM_QUERYENDSESSION)
		{
			this.mdiclient.CloseAll()
		}
		return base.OnMessage(hWnd, uMsg, wParam, lParam)
	}
	DefaultWindowProc(hWnd, uMsg, wParam, lParam)
	{
		if (!this.mdiclient.hWnd)
		{
			this.LogWindowMessage(A_ThisFunc, uMsg, wParam, lParam)
		}
		return DllCall("DefFrameProc", "ptr", hWnd, "ptr", this.mdiclient.hWnd, "UInt" ,uMsg, "UPtr",wParam, "Ptr",lParam)
	}
	MessageLoop()
	{
		VarSetCapacity(msg, A_PtrSize == 8 ? 48 : 28, 0)
		while(DllCall("GetMessage", "ptr", &msg, "ptr", 0, "uint", 0, "uint", 0))
		{
			if (!DllCall("TranslateMDISysAccel", "ptr", this.mdiclient.hWnd, "ptr", &msg))
			{
				DllCall("TranslateMessage", "ptr", &msg)
				DllCall("DispatchMessage", "ptr", &msg)
			}
		}
	}

}
class CMDIClientWindow extends CWindow
{
	static MC_CLOSEALL := 0, MC_RESTOREALL := 1
	__New(hWndParent, hWindowMenu)
	{
		base.__New()
		this.hWndParent := hWndParent
		this.hWindowMenu := hWindowMenu
		this.mdichild := new CMDIChildWindow("AHK_MDI_CHILD", "MDI Child")
	}
	SetCreateWindowInfo()
	{
		static clientcreate
		this.cwex.szClassName := "MDICLIENT"
		this.cwex.dwStyle := CCWEX.WS_CHILD  | CCWEX.WS_CLIPCHILDREN | CCWEX.WS_VISIBLE
		IDM_FIRSTCHILD := 50000
		hWindowMenu := this.hWindowMenu
		idFirstChild := IDM_FIRSTCHILD
		VarSetCapacity(clientcreate, A_PtrSize == 8 ? 16 : 8, 0)
		NumPut(hWindowMenu, clientcreate, 0, "Ptr")
		NumPut(idFirstChild, clientcreate, A_PtrSize == 8 ? 8 : 4, "UInt")
		this.cwex.hWndParent := this.hWndParent
		this.cwex.lpParam := &clientcreate
	}
	Activate(hWndChild)
	{
		return this.SendMessage(CWindowMessage.WM_MDIACTIVATE, hWndChild, 0)
	}
	Cascade()
	{
		return this.SendMessage(CWindowMessage.WM_MDICASCADE, 0, 0)
	}
	CreateChild(szTitle := "")
	{
		szClass := this.mdichild.wcex.szClassName
		if (!szTitle)
		{
			szTitle := this.mdichild.wcex.szWindowName
		}
		hOwner := 0
		x := CCWEX.CW_USEDEFAULT
		y := CCWEX.CW_USEDEFAULT
		cx := CCWEX.CW_USEDEFAULT
		cy := CCWEX.CW_USEDEFAULT
		style := 0
		lParam := 0
		VarSetCapacity(mdicreate, A_PtrSize == 8 ? 56 : 36, 0)
		NumPut(&szClass, mdicreate, 0, "Ptr")
		NumPut(&szTitle, mdicreate, A_PtrSize == 8 ? 8 : 4, "Ptr")
		NumPut(hOwner, mdicreate, A_PtrSize == 8 ? 16 : 8, "Ptr")
		NumPut(x, mdicreate, A_PtrSize == 8 ? 24 : 12, "Int")
		NumPut(y, mdicreate, A_PtrSize == 8 ? 28 : 16, "Int")
		NumPut(cx, mdicreate, A_PtrSize == 8 ? 32 : 20, "Int")
		NumPut(cy, mdicreate, A_PtrSize == 8 ? 36 : 24, "Int")
		NumPut(style, mdicreate, A_PtrSize == 8 ? 40 : 28, "UInt")
		NumPut(lParam, mdicreate, A_PtrSize == 8 ? 48 : 32, "Ptr")
		hWndChild := this.SendMessage(CWindowMessage.WM_MDICREATE, 0, &mdicreate)
		this.mdichild.ListenMessage(hWndChild)
		this.Activate(hWndChild)
		return hWndChild
	}
	CloseAll()
	{
		this.EnumChildWindows(this.MC_CLOSEALL)
		return true
	}
	Destory(hWndChild)
	{
		return this.SendMessage(CWindowMessage.WM_MDIDESTROY, hWndChild, 0)
	}
	GetActive()
	{
		return this.SendMessage(CWindowMessage.WM_MDIGETACTIVE, 0, 0)	
	}
	IconArrange()
	{
		return this.SendMessage(CWindowMessage.WM_MDIICONARRANGE, 0, 0)
	}
	RefreshMenu()
	{
		return this.SendMessage(CWindowMessage.WM_MDIREFRESHMENU, 0, 0)
	}
	Restore(hWndChild)
	{
		return this.SendMessage(CWindowMessage.WM_MDIRESTORE, hWndChild, 0)
	}
	SetMenu(hMenu, hWindowMenu)
	{
		return this.SendMessage(CWindowMessage.WM_MDISETMENU, hMenu, hWindowMenu)
	}
	Tile()
	{
		return this.SendMessage(CWindowMessage.WM_MDITILE, 0, 0)
	}
	OnChildWindow(hWnd, lParam)
	{
		LogMessage("child=" hWnd ", lParam=" lParam)
		if (lParam = this.MC_CLOSEALL)
		{
			GW_OWNER := 4
			child := new CWindow(hWnd)
			if (child.GetWindow(GW_OWNER))
			{
				return true
			}
			this.Restore(hWnd)
			if (!child.SendMessage(CWindowMessage.WM_QUERYENDSESSION, 0, 0))
			{
				return true
			}
			this.Destory(hWnd)
			return true
		}
		return true
	}
	OnMessage(hWnd, uMsg, wParam, lParam)
	{
		static __ := CWindowMessage
		return 0
	}
}
class CMDIChildWindow extends CWindow
{
	__New(szClassName, szWindowName)
	{
		TRACE(A_LineFile, A_LineNumber, A_Thisfunc)
		base.__New()
		this.RegisterEx(szClassName, szWindowName)
	}
	SetWindowClassInfo()
	{
		IDC_ARROW := 32512
		WHITE_BRUSH := 0
		this.wcex.hCursor := DllCall("LoadCursor", "ptr", 0, "ptr", IDC_ARROW)
		this.wcex.hbrBackground := DllCall("GetStockObject", "int", WHITE_BRUSH)
	}
	OnMessage(hWnd, uMsg, wParam, lParam)
	{
		static __ := CWindowMessage
		static parent := 0
		static frame := 0
		if (uMsg = __.WM_MDIACTIVATE)
		{
			if (!parent)
			{
				parent := new CWindow(this.GetParent())
				frame := new CWindow(parent.GetParent())
			}
			frame.DrawMenuBar()
			return false
		}
		else if (uMsg = __.WM_PAINT)
		{
			VarSetCapacity(ps, A_PtrSize == 8 ? 72 : 64, 0)
			hdc := DllCall("BeginPaint", "ptr", hWnd, "ptr", &ps)
			DllCall("EndPaint", "ptr", hWnd, "ptr", &ps)
		}
		return 0
	}
	DefaultWindowProc(hWnd, uMsg, wParam, lParam)
	{
		return DllCall("DefMDIChildProc", "ptr", hWnd, "UInt" ,uMsg, "UPtr",wParam, "Ptr",lParam)
	}
}



global _app := new CMDIFrameWindow("AHK_MDI_MAINFRAME", "Autohotkey MDI Application Example")
_app.CreateEx()
_app.ShowWindow()
_app.UpdateWindow()
;  #Persistent 를 쓰면 오토핫키엔진에서 메세지루프를 돌리므로 아래는 필요없음
;_app.MessageLoop()
